package fxhevonen;

import java.net.URL;
import java.util.ResourceBundle;

import fi.jyu.mit.fxgui.StringGrid;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.TextArea;
import javafx.scene.input.KeyCode;

/**
 * Yksikertaistettu risitinolla. 
 * Vuorossa oleva pelaaja nytetn otsikko-rivin 0-sarakkeessa.
 * Alueen oikean reunaan nytetn siirtohistoria
 * @author vesal
 * @version 26.3.2016
 */
public class HevosenHyppelyController implements Initializable {
    @FXML private StringGrid<Object> grid;
    @FXML private TextArea area;
    private int vuorossa = 1;
    private int kokoX = 5;
    private int kokoY = 7;
    
    
    @Override
    public void initialize(URL location, ResourceBundle resources) {
        //grid.setEmptyStyleClass("valkoinen"); // tyhjt solut valkoiseksi // voi hoitaa css:ll
        //grid.setColumnResizePolicy(TableView.CONSTRAINED_RESIZE_POLICY); // ylim. sarake piiloon
        grid.getSelectionModel().setCellSelectionEnabled(true); // yksittinen solu voidaan valita
        String[] headings = new String[kokoX+1];
        String[] tyhja = new String[kokoX+1];
        headings[0] = "";
        for (int i=1; i<=kokoX; i++) headings[i] = ""+i;
        grid.initTable(headings); // Alustetaan otsikot juoksevilla numeroilla
        for (int i=0; i<kokoX+1; i++) tyhja[i] = "";
        
        grid.setColumnWidth(-1,30);   // sarakkeet kapeammiksi
        grid.setSortable(-1, false);  // jrjestminen ei ole mielekst
        grid.disableColumnReOrder();
        grid.setPrefWidth(30*(kokoX+1)+2); // pakotetaan koko
        
        for (int i=1; i<=kokoY; i++) { // listn rivinumero ja rivit
            tyhja[0] = ""+i;
            grid.add(tyhja);
            grid.setStyleClass("otsikko", i-1, 0);
        }
            
        naytaVuorossa();
        
        grid.setOnMouseClicked( e -> laitaMerkki() );
        grid.setOnKeyPressed( e -> {if ( e.getCode() == KeyCode.ENTER ) laitaMerkki();});
    }
    

    private void laitaMerkki() {
        int r = grid.getRowNr();
        int c = grid.getColumnNr();
        if ( r < 0 || c < 1 ) return; // otsikkoon ei saa klikata
        String s = grid.get(r, c); 
        if ( !s.isEmpty() ) { // vain tyhjn saa laittaa
            if ( Integer.parseInt(s)+1 != vuorossa) return; // tai edellisen poistaa
            vuorossa--;
            grid.setStyleClass("kayty", r, c);
            grid.set("", r, c);
            naytaVuorossa();
            return;
        }
        grid.setStyleClass("hepo", r, c);
        String merkki = ""+vuorossa;
        grid.set(merkki, r, c);
        area.setText(area.getText() + String.format("%s -> (%d,%d)\n", merkki, r+1, c));
        vuorossa++;
        naytaVuorossa();
    }

    
    private void naytaVuorossa() {
        grid.getColumns().get(0).setText(""+vuorossa);
    }
    
}