package fi.jyu.mit.fxgui;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;

import javafx.scene.Node;
import javafx.scene.layout.Pane;

/**
 * Apufunktioita JavaFX-olioiden käsittelyyn
 * @author vesal
 * @version 6.3.2016
 */
public class Functions {
    /**
     * Etsii kaikki ehdot täyttävät elementit containerin sisältä 
     * @param parent containeri, josta etsitään.  Tätä ei lisätä listaan vaikka täyttäisi ehdon
     * @param type mitä tyyppiä elementin on oltavat, esimerkksi Label.class
     *             jos halutaan kaikki, voi käyttää Node.class
     * @param condition ehto, jolla elementti hyväksytään, esimerkkisi n -> "HA".equals(n.getId())
     * @param recursive etsitäänkö rekursiivisesti
     * @return lista ehdon täyttävistä elementeistä
     */
    @SuppressWarnings("unchecked")
    public static <T> List<T> getNodes(Node parent, Class<T> type, Predicate<Node> condition, boolean recursive) {
        List<T> elements = new ArrayList<>();
            
        if ( !(parent instanceof Pane) ) return elements;
        
        for (Node node : ((Pane)parent).getChildren()) {
            if (type.isAssignableFrom(node.getClass()) && condition != null && condition.test(node) ) 
                elements.add((T) node);
            if (node instanceof Pane && recursive) 
                elements.addAll(getNodes((Pane) node, type, condition, recursive));
        }
        return elements;
    }
    
    
    /**
     * Etsii ensimmäisen ehdon täyttävän elementin containerin sisältä 
     * @param parent containeri, josta etsitään.  Tätä ei palauteta vaikka täyttäisi ehdon
     * @param type mitä tyyppiä elementin on oltavat, esimerkksi Label.class
     *             jos halutaan kaikki, voi käyttää Node.class
     * @param condition ehto, jolla elementti hyväksytään, esimerkkisi n -> "HA".equals(n.getId())
     * @param recursive etsitäänkö rekursiivisesti
     * @return ehdon täyttävä elementti tai null jos ei löydy
     */
    @SuppressWarnings("unchecked")
    public static <T> T getNode(Node parent, Class<T> type, Predicate<Node> condition, boolean recursive) {
        if ( !(parent instanceof Pane) ) return null;
        
        for (Node node : ((Pane)parent).getChildren()) {
            if (type.isAssignableFrom(node.getClass()) && condition != null && condition.test(node) ) 
                return (T)node;
            if (node instanceof Pane && recursive) { 
                T result = getNode((Pane) node, type, condition, recursive);
                if ( result != null ) return result;
            }    
        }
        return null;
    }

    
    /**
     * Poistetaan alleviiva
     * @param item mistä poistetaan
     * @return ilman 1. alleviivaa
     */
    public static String removeMnemonic(String item) {
        return item.replaceFirst("_", "");
    }
}
