package fi.jyu.mit.fxgui;

import java.io.IOException;
import java.net.URL;

import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Modality;
import javafx.stage.Stage;

/**
 * Modaalisen dialogin näyttäminen
 * 
 * @author vesal
 * @version 2.1.2016
 * @param <TYPE> minkä tyyppinen tulos palautetaan dialogista.
 */
public class ModalController<TYPE> implements ModalControllerInterface<TYPE>  {
    
    /**
     * Funktionaalinen rajapinta takaisinkutsulle luokan alustamiseksi 
     * @author vesal
     * @version 18.3.2016
     *
     * @param <TYPE> mikä on perusluokan tyyppi
     * @param <CONTROLLERTYPE> mikä on dialogin kontrolleriluokan tyyppi
     */
    public interface ModalInitializeInterface<TYPE, CONTROLLERTYPE extends ModalControllerInterface<TYPE>> {
        /**
         * @param ctrl kontrolleri, jonne jotakin asetetaan
         */
        public void initialize(CONTROLLERTYPE ctrl);
    }
    
    
    class MyGenericClass<T> {
        private final Class<T> clazz;
        public MyGenericClass(Class<T> clazz) {
          this.clazz = clazz;
        }
        public Class<T> getGenericClass() {
          return clazz;
        }
      }

    
    private Stage stage;
    
    @FXML private void handleDefaultOK() {
        stage.close();
    }
    
    
    @FXML private void handleDefaultCancel() {
        stage.close();
    }
    
    
    /**
     * @return palautetaan dialogin tulos
     */
    @Override
    public TYPE getResult() {
        return null;
    }


    /**
     * Asetetaan oletusarvo
     * @param oletus mitä näyteteään oletuksena
     */
    @Override
    public void setDefault(TYPE oletus) {
        //
    }

    
    /**
     * Mitä tehdään kun dialogi on näytetty
     */
    @Override
    public void handleShown() {
        //
    }

    
    /**
     * Palauttaa nodea vastaavan stagen
     * @param n minkä stagea etsitään
     * @return nodea vstaava stage
     */
    public static Stage getStage(Node n) {
        return ((Stage)n.getScene().getWindow());
    }
    
    
    /**
     * Sulkee stagen johon node kuuluu
     * @param n minkä stagea etsitään
     */
    public static void closeStage(Node n) {
         getStage(n).close();
    }
    
    
    /**
     * Luodaan dialogi ja palautetaan siihen täytetty data tai null
     * @param url osoite, josta .fxml tiedosto löytyy
     * @param title Otsikko joka näytetään
     * @param modalityStage mille ollaan modaalisia, null = sovellukselle
     * @param oletus mitä dataa näytetään oletuksena
     * @return null jos painetaan Cancel, muuten kirjoitettu nimi
     */
    public static <TYPE> TYPE showModal(URL url, String title, Stage modalityStage, TYPE oletus) {
        return showModal(url, title, modalityStage, oletus, null);
    }
    
    
    /**
     * Luodaan dialogi ja palautetaan siihen täytetty data tai null
     * @param url osoite, josta .fxml tiedosto löytyy
     * @param title Otsikko joka näytetään
     * @param modalityStage mille ollaan modaalisia, null = sovellukselle
     * @param oletus mitä dataa näytetään oletuksena
     * @param initializeCallback mitä kutsutaan kun dialogi on luotu
     * @return null jos painetaan Cancel, muuten dialogin data
     */
    public static <TYPE, CONTROLLERTYPE extends ModalControllerInterface<TYPE>> 
            TYPE showModal(URL url, String title, Stage modalityStage, TYPE oletus,
            ModalInitializeInterface<TYPE, CONTROLLERTYPE> initializeCallback) {
        try {
            FXMLLoader ldr = new FXMLLoader(url);
            Parent root =  ldr.load();
            @SuppressWarnings("unchecked")
            final CONTROLLERTYPE dialogCtrl = (CONTROLLERTYPE)ldr.getController();
            Stage stage = new Stage();
            if ( dialogCtrl instanceof ModalController<?> ) {
                ((ModalController<?>)dialogCtrl).stage = stage; 
            }
            
            stage.setScene(new Scene(root));
            stage.setTitle(title);
            if ( modalityStage != null ) {
                stage.initModality(Modality.WINDOW_MODAL);
                stage.initOwner(modalityStage);
            } else    
                stage.initModality(Modality.APPLICATION_MODAL);
            
            if ( dialogCtrl != null ) {
                dialogCtrl.setDefault(oletus);
                stage.setOnShown((e)-> {
                    // dialogCtrl.vastaus.requestFocus();
                    //dialogCtrl.vastaus.selectAll();
                    dialogCtrl.handleShown();
                });
                if ( initializeCallback != null ) initializeCallback.initialize(dialogCtrl);
                stage.showAndWait();
                return dialogCtrl.getResult();
            }
            stage.showAndWait();
            return null;
            
        } catch (IOException e) {
            System.err.println(e.toString());
            return null;
        }
    }
    
    
    /**
     * Luodaan modeless dialogi ja palautetaan sen käsittelijä
     * @param url osoite, josta .fxml tiedosto löytyy
     * @param title Otsikko joka näytetään
     * @param oletus mitä dataa näytetään oletuksena
     * @return null jos painetaan Cancel, muuten kirjoitettu nimi
     */
    public static <TYPE, CONTROLLERTYPE extends ModalControllerInterface<TYPE>> CONTROLLERTYPE  showModeless(URL url, String title, TYPE oletus) {
        Stage stage = new Stage();
        try {
            FXMLLoader ldr = new FXMLLoader(url);
            Parent root =  ldr.load();
            @SuppressWarnings("unchecked")
            final CONTROLLERTYPE dialogCtrl = (CONTROLLERTYPE)ldr.getController();
            stage.setScene(new Scene(root));
            stage.setTitle(title);
            if ( dialogCtrl != null ) {
                dialogCtrl.setDefault(oletus);
                stage.setOnShown((e)-> {
                    // dialogCtrl.vastaus.requestFocus();
                    //dialogCtrl.vastaus.selectAll();
                    dialogCtrl.handleShown();
                });
                stage.show();
                return dialogCtrl;
            }
            stage.show();
            return null;
            
        } catch (IOException e) {
            System.err.println(e.toString());
            return null;
        }
    }
}
