package javafxmlpackage.wizards;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.operation.*;
import java.lang.reflect.InvocationTargetException;
import java.util.Calendar;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.core.resources.*;

import java.io.*;
import org.eclipse.ui.*;
import org.eclipse.ui.ide.IDE;

/**
 * 
 * Wizard, joka luo jyu ohj2 kurssin mukaisen javaFXML packagen.
 */

public class PackageWizard extends Wizard implements INewWizard {
    private PackageWizardPage page;
    private ISelection selection;

    /**
     * Constructor for PackageWizard.
     */
    public PackageWizard() {
        super();
        setNeedsProgressMonitor(true);
    }

    /**
     * Adding the page to the wizard.
     */

    @Override
    public void addPages() {
        page = new PackageWizardPage(selection);
        addPage(page);
    }

    /**
     * This method is called when 'Finish' button is pressed in
     * the wizard. We will create an operation and run it
     * using wizard as execution context.
     */
    @Override
    public boolean performFinish() {
        final String containerName = page.getContainerName();
        final String fileName = page.getFileName();
        final String packageName = page.getPackageName();

        IRunnableWithProgress op = new IRunnableWithProgress() {
            @Override
            public void run(IProgressMonitor monitor) throws InvocationTargetException {
                try {
                    doFinish(containerName, packageName, fileName, monitor);
                } catch (CoreException e) {
                    throw new InvocationTargetException(e);
                } finally {
                    monitor.done();
                }
            }
        };
        try {
            getContainer().run(true, false, op);
        } catch (InterruptedException e) {
            return false;
        } catch (InvocationTargetException e) {
            Throwable realException = e.getTargetException();
            MessageDialog.openError(getShell(), "Error", realException.getMessage());
            return false;
        }
        return true;
    }

    /**
     * 
     * Luodaan tiedostot
     * @param containerName projektin src folder
     * @param packageName paketin nimi
     * @param fileName ohjelman nimi
     * @param monitor eclipse monitori
     * @throws CoreException eclipse joku core exception varmaanki
     * 
     */

    protected void doFinish(
            String containerName,
            String packageName,
            String fileName,
            IProgressMonitor monitor)
                    throws CoreException {
        // create a sample file

        String parsePackageName = packageName.replace('.', '/');

        String parseFileName = fileName;
        
        String vaihdaKirjain = ""; // niden on syyt olla yht pitki
        String uusiKirjain =   "AAOaao"; // tai kyttj saa vastaansa exceptioni
        								 // ja ohi psee korjaamalla ite kkset pois nimest,
        								 // joten ei est pluginin kyttmist, vaikka joku hlmilisikin

        for (int i = 0; i < vaihdaKirjain.length(); i++) {
        	parsePackageName = parsePackageName.replace(vaihdaKirjain.charAt(i), uusiKirjain.charAt(i));
            parseFileName = parseFileName.replace(vaihdaKirjain.charAt(i), uusiKirjain.charAt(i));
        }
        
     // siivotaa mahdolliset tuplamerkit yksinkertasella algoritmil... ei jaksa miettii mit
        StringBuilder sb = new StringBuilder(parsePackageName);
        
        while (sb.indexOf("//") != -1) {
            sb.deleteCharAt(sb.indexOf("//"));
        }

        parsePackageName = sb.toString();



        if (Character.isLowerCase(parseFileName.charAt(0))) 
            parseFileName = Character.toUpperCase(parseFileName.charAt(0)) + parseFileName.substring(1);

        monitor.beginTask("Creating files...", 2);
        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
        IResource resource = root.findMember(new Path(containerName));
        if (!resource.exists() || !(resource instanceof IContainer)) {
            throwCoreException("Project \"" + containerName + "\" does not exist.");
        }

        if (resource.getProjectRelativePath().isEmpty()) {
            throwCoreException("src folder ei kelpaa");
        }


        IContainer container = (IContainer) resource;
        final IFile javaFile = container.getFile(new Path(parsePackageName +"/" +parseFileName + "Main.java"));
        final IFile controllerFile = container.getFile(new Path(parsePackageName +"/" +parseFileName+ "Controller.java"));
        final IFile cssFile = container.getFile(new Path(parsePackageName +"/" +parseFileName.toLowerCase()+ ".css"));
        final IFile viewFile = container.getFile(new Path(parsePackageName +"/" +parseFileName+ "View.fxml"));


        if (parsePackageName.lastIndexOf('/') == -1) {
            final IFolder packageF = container.getFolder(new Path(parsePackageName));
			if (!packageF.exists())
				packageF.create(false, false, monitor);
        }

        else {
            String[] splitti = parsePackageName.split("/");
            String pathi = "/";
            for (int i = 0; i < splitti.length; i++) {
                if (splitti[i].length() == 0) continue;
                pathi += splitti[i] + "/";
                final IFolder packageF = container.getFolder(new Path(pathi));
                if (!packageF.exists())
                    packageF.create(false, false, monitor);
            }
        }


        try (InputStream stream = openContentStream(1, parsePackageName, parseFileName, fileName)){
            cssFile.create(stream, true, monitor);

        } catch (IOException e) {
            //
        }
        
        try (InputStream stream = openContentStream(2, parsePackageName, parseFileName, fileName)){
            controllerFile.create(stream, true, monitor);

        } catch (IOException e) {
            //
        }
        
        try (InputStream stream = openContentStream(3, parsePackageName, parseFileName, fileName)){
            javaFile.create(stream, true, monitor);

        } catch (IOException e) {
            //
        }
        
        try (InputStream stream = openContentStream(4, parsePackageName, parseFileName, fileName)){
            viewFile.create(stream, true, monitor);

        } catch (IOException e) {
            //
        }

        // tarkistetaan, ett on javafx projekti
        tarkistaProjekti(resource.getParent(), monitor);
        
        monitor.worked(1);
        monitor.setTaskName("Opening file for editing...");
        getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                IWorkbenchPage sivu =
                        PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
                try {
                    IDE.openEditor(sivu, javaFile, true);
                } catch (PartInitException e) {
                    //
                }
            }
        });
        monitor.worked(1);
    }
    
    private void tarkistaProjekti(IContainer cont, IProgressMonitor monitor) throws CoreException {
        //
        if (!cont.getProjectRelativePath().isEmpty()) return; // TODO: ollaan jossai muualla kuin projektin juuressa..
        final IFile fxbuildFile = cont.getFile(new Path("/build.fxbuild"));
        if (fxbuildFile.exists()) {
            // javafx projekti, joten lhet pois
            return;
        }
        try (InputStream stream = openFxprojektStream(cont)){
            fxbuildFile.create(stream, true, monitor);

        } catch (IOException e) {
            //
        }

        // muokataa .project filua

        final IFile projectFile = cont.getFile(new Path(".project"));
        if (!fxbuildFile.exists()) {
            // .projectia ei ole, toivottavasti kyttj tiet mit on tekemss...
            return;
        }

        try (InputStream stream = projectFile.getContents()) {
            //
            StringBuilder sisalto = getStringBuilder(stream);

            String mihin = "</buildSpec>";
            int index = sisalto.lastIndexOf(mihin);
            if (index == -1) return; 
            String lisattava = 
                    "    <buildCommand>"+System.lineSeparator()+
                    "            <name>org.eclipse.xtext.ui.shared.xtextBuilder</name>"+System.lineSeparator()+
                    "            <arguments>"+System.lineSeparator()+
                    "            </arguments>"+System.lineSeparator()+
                    "        </buildCommand>"+System.lineSeparator()+"\t";
            sisalto.insert(index, lisattava);
            
            mihin = "<nature>";
            index = sisalto.lastIndexOf(mihin);
            if (index == -1) return; 
            lisattava = 
                    "<nature>org.eclipse.xtext.ui.shared.xtextNature</nature>"+System.lineSeparator()+"\t\t";
            sisalto.insert(index, lisattava);
            projectFile.setContents(new ByteArrayInputStream(sisalto.toString().getBytes()), true, true, monitor);
        } catch (IOException e) {
            //
        }
        
        // muokataan .classpathia
        final IFile classpathFile = cont.getFile(new Path(".classpath"));
        if (!fxbuildFile.exists()) {
            // .classpathia ei ole, toivottavasti kyttj tiet mit on tekemss...
            return;
        }
        try (InputStream stream = classpathFile.getContents()) {
            //
            StringBuilder sisalto = getStringBuilder(stream);
            
            String mihin = "<classpathentry";
            int index = sisalto.lastIndexOf(mihin);
            if (index == -1) return; 
            String lisattava = 
                    "<classpathentry kind=\"con\" path=\"org.eclipse.fx.ide.jdt.core.JAVAFX_CONTAINER\"/>"+System.lineSeparator()+"\t";
            sisalto.insert(index, lisattava);
            
            classpathFile.setContents(new ByteArrayInputStream(sisalto.toString().getBytes()), true, true, monitor);
        } catch (IOException e) {
            //
        }
        
    }
    
    /**
     * muutetaa inputstream stringiksi, jotta voidaan muokkaa sit
     * @param stream is
     * @return is -> string
     * @throws IOException 
     */
    private StringBuilder getStringBuilder(InputStream stream) throws IOException {
        //
        StringBuilder sb = new StringBuilder();
        String s;
        try (BufferedReader br = new BufferedReader(new InputStreamReader(stream))) {
            while ((s = br.readLine()) != null) {
                // joudutaa lism rivinvaihto uusiks, mutta tm varmaa helpoin tapa knt inpustream stringiks
                sb.append(s + System.lineSeparator());
            }
        }
        return sb;
    }

    /**
     * Filujen sislt... t ny siin plugin wizardis oli tehty tllei, ett teh samalla tavalla 
     * pienell omalla jollai.. - iha varmuuden vuoks
     * 
     * monesko - 1 = .css tiedosto, 2 = Controller tiedosto, 3 = .java tiedosto, 4 = xml tiedosto
     */

    private InputStream openContentStream(int monesko, String packageName, String fileName, String alkupFileName) {

        Calendar aika = Calendar.getInstance();


        String contents = "";
        String parsePackageName = packageName.replace('/', '.');
        switch (monesko) {

        case 1: contents = "/* JavaFX CSS - Leave this comment until you have at least create one rule which uses -fx-Property */";
        break;
        case 2: 
            contents = 
            "package "+parsePackageName+";"+System.lineSeparator()+
            System.lineSeparator()+
            "/**"+System.lineSeparator()+
            " * @author "+System.getProperty("user.name").toString()+System.lineSeparator()+
            " * @version "+aika.get(Calendar.DAY_OF_MONTH)+"."+(aika.get(Calendar.MONTH) + 1)+"."+aika.get(Calendar.YEAR)+System.lineSeparator()+
            " *"+System.lineSeparator()+
            " */"+System.lineSeparator()+
            "public class "+fileName+"Controller {"+System.lineSeparator()+
            "      // TODO"+System.lineSeparator()+
            "}";

            break;
        case 3: contents =
                "package "+parsePackageName+";"+System.lineSeparator()+System.lineSeparator()+

                "import javafx.application.Application;" +System.lineSeparator()+
                "import javafx.stage.Stage;"+System.lineSeparator()+
                "import javafx.scene.Scene;"+System.lineSeparator()+
                "import javafx.scene.layout.Pane;"+System.lineSeparator()+
                "import javafx.fxml.FXMLLoader;"+System.lineSeparator()+System.lineSeparator()+System.lineSeparator()+


                "/**"+System.lineSeparator()+
                " * @author "+System.getProperty("user.name").toString()+System.lineSeparator()+
                " * @version "+aika.get(Calendar.DAY_OF_MONTH)+"."+(aika.get(Calendar.MONTH) + 1)+"."+aika.get(Calendar.YEAR)+System.lineSeparator()+
                " *"+System.lineSeparator()+
                " */"+System.lineSeparator()+
                "public class "+fileName+"Main extends Application {"+System.lineSeparator()+
                "    @Override"+System.lineSeparator()+
                "    public void start(Stage primaryStage) {"+System.lineSeparator()+
                "        try {"+System.lineSeparator()+
                "            FXMLLoader ldr = new FXMLLoader(getClass().getResource(\""+fileName+"View.fxml\"));"+System.lineSeparator()+
                "            final Pane root = ldr.load();"+System.lineSeparator()+
                "            //final "+fileName+"Controller "+fileName.toLowerCase()+"Ctrl = ("+fileName+"Controller) ldr.getController();"+System.lineSeparator()+
                "            Scene scene = new Scene(root);"+System.lineSeparator()+
                "            scene.getStylesheets().add(getClass().getResource(\""+fileName.toLowerCase()+".css\").toExternalForm());"+System.lineSeparator()+
                "            primaryStage.setScene(scene);"+System.lineSeparator()+
                "            primaryStage.setTitle(\""+alkupFileName+"\");"+System.lineSeparator()+
                "            primaryStage.show();"+System.lineSeparator()+
                "        } catch(Exception e) {"+System.lineSeparator()+
                "            e.printStackTrace();"+System.lineSeparator()+
                "        }"+System.lineSeparator()+
                "    }"+System.lineSeparator()+System.lineSeparator()+

                "    /**"+System.lineSeparator()+
                "     * @param args Ei kytss"+System.lineSeparator()+
                "     */"+System.lineSeparator()+
                "    public static void main(String[] args) {"+System.lineSeparator()+
                "        launch(args);"+System.lineSeparator()+
                "    }"+System.lineSeparator()+
                "}"; break;


        case 4:
            contents = 
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +System.lineSeparator()+System.lineSeparator()+

            "<?import javafx.scene.layout.BorderPane?>"+System.lineSeparator()+System.lineSeparator()+

            "<BorderPane xmlns:fx=\"http://javafx.com/fxml/1\" fx:controller=\""+parsePackageName+"."+fileName+"Controller\">"+System.lineSeparator()+
            "    <!-- TODO Add Nodes -->"+System.lineSeparator()+
            "</BorderPane>"+System.lineSeparator();
            break;
        default:
            break;
        }
        return new ByteArrayInputStream(contents.getBytes());
    }
    
    private InputStream openFxprojektStream(IContainer cont) {
        String contents = 
                "<?xml version=\"1.0\" encoding=\"ASCII\"?>"+System.lineSeparator()+
                "<anttasks:AntTask xmi:version=\"2.0\" xmlns:xmi=\"http://www.omg.org/XMI\" xmlns:anttasks=\"http://org.eclipse.fx.ide.jdt/1.0\" buildDirectory=\"${project}/build\">"+System.lineSeparator()+
                "  <deploy>"+System.lineSeparator()+
                "    <application name=\""+cont.getFullPath().toString().replace("/","")+"\"/>"+System.lineSeparator()+
                "    <info/>"+System.lineSeparator()+
                "  </deploy>"+System.lineSeparator()+
                "  <signjar/>"+System.lineSeparator()+
                "</anttasks:AntTask>"+System.lineSeparator();
        return new ByteArrayInputStream(contents.getBytes());
    }

    private void throwCoreException(String message) throws CoreException {
        IStatus status =
                new Status(IStatus.ERROR, "javaFXpackage", IStatus.OK, message, null);
        throw new CoreException(status);
    }

    /**
     * We will accept the selection in the workbench to see if
     * we can initialize from it.
     * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
     */
    @Override
    public void init(IWorkbench workbench, IStructuredSelection select) {
        this.selection = select;
    }
}