package javafxmlpackage.wizards;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;

/**
 * Wizard, joka luo jyu ohj2 kurssin mukaisen javaFXML packagen.
 * 
 */

public class PackageWizardPage extends WizardPage {
	private Text containerText;
	private Text packageName;
	private Text fileName;
	private ISelection selection;

	/**
	 * Constructor for PackageNewWizardPage.
	 * @param selection s

	 */
	public PackageWizardPage(ISelection selection) {
		super("wizardPage");
		setTitle("Uusi JavaFXML Package");
		setDescription("Luo uusi package");
		this.selection = selection;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	@Override
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);

		GridLayout layout = new GridLayout();

		container.setLayout(layout);
		layout.numColumns = 3;

		layout.verticalSpacing = 9;
		Label label = new Label(container, SWT.NULL);
		label.setText("&Project src folder:");

		containerText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		containerText.setLayoutData(gd);

		containerText.addModifyListener( e -> dialogChanged());


		Button button = new Button(container, SWT.PUSH);
		button.setText("Browse...");

		button.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				handleBrowse();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("&Packagen nimi:");
		packageName = new Text(container, SWT.BORDER | SWT.SINGLE);

		gd = new GridData(GridData.FILL_HORIZONTAL);
		packageName.setLayoutData(gd);
		packageName.addModifyListener( e -> dialogChanged());


		label = new Label(container, SWT.NULL); // tyhj labeli, koska gridiss 3 saraketta

		label = new Label(container, SWT.NULL);
		label.setText("&Ohjelman nimi:");

		fileName = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		fileName.setLayoutData(gd);

		fileName.addModifyListener( e -> dialogChanged());

		initialize();
		dialogChanged();
		setControl(container);
	}

	/**
	 * Tests if the current workbench selection is a suitable container to use.
	 */

	private void initialize() {
		if (selection != null && selection.isEmpty() == false
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();

			if (obj instanceof IResource) {
				IContainer container;
				String pathi = "";
				if (obj instanceof IContainer) {
					container = (IContainer) obj;
					pathi = container.getFullPath().toString();
					if (!pathi.matches(".*/src/"))
						pathi = container.getFullPath().toString() +"/src/";
				}
				else {
					container = ((IResource) obj).getParent();
					pathi = container.getFullPath().toString();

					Pattern p = Pattern.compile("^(.*/src)(.*)");
					Matcher m = p.matcher(pathi);
					if (m.find()) {
						pathi = m.group(1);
						if (!m.group(2).isEmpty()) {
							String mg = m.group(2).replace('/', '.');
							packageName.setText(m.group(2).replace('/', '.'));
							if (mg.charAt(0) == '.')
								packageName.setText(mg.substring(1));
							else packageName.setText(mg);
						}
					}
				}

				containerText.setText(pathi);
			}
			else {
				// toi objecti voi ny olla mys joku class org.eclipse.jdt.internal.core.* classi
				// joihin ei ny pse ksiks, joten yritet arvata sen palauttamasta stringist miss mahtais source folderi olla :)

				String stringi = obj.toString();


				Pattern p = Pattern.compile("(.*)src \\[in ([^\\]]+)\\]*.");
				Matcher m = p.matcher(stringi);

				if (m.find()) {
					containerText.setText("/"+m.group(2)+"/src/");
					if (!m.group(1).isEmpty()) {
						String[] splitti = m.group(1).split(" ");
						// jooh eli ku katotaa stringist vaa tota pathia, nii pit karsii erikoistapauksia pois 
						// manuaalisesti......... ei hyv
						if (!splitti[0].matches(".*\\.java$") && !splitti[0].matches("^\\[.*"))
							packageName.setText(splitti[0]);
					}
				}

			}
		}
	}

	/**
	 * Uses the standard container selection dialog to choose the new value for
	 * the container field.
	 */

	protected void handleBrowse() {
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(
				getShell(), ResourcesPlugin.getWorkspace().getRoot(), false,
				"Select project src folder");
		if (dialog.open() == Window.OK) {
			Object[] result = dialog.getResult();
			if (result.length == 1) {
				containerText.setText(((Path) result[0]).toString());
			}
		}
	}

	/**
	 * Valitetaan virheist jne..
	 */

	private void dialogChanged() {
		IResource container = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(new Path(getContainerName()));
		String fileN = getFileName();
		String packageN = getPackageName();

		if (getContainerName().length() == 0) {
			updateStatus("Projektin src folderi puuttuu");
			return;
		}

		if (container == null
				|| (container.getType() & (IResource.PROJECT | IResource.FOLDER)) == 0) {
			updateStatus("Projektin src folderi puuttuu");
			return;
		}
		if (!container.isAccessible()) {
			updateStatus("Ei oikeuksia projektin src folderiin");
			return;
		}
		if (fileN.length() == 0 || packageN.length() == 0) {
			updateStatus("Paketin tai ohjelman nimi mrittmtt");
			return;
		}


		if (!Pattern.matches("[a-zA-Z0-9]+", fileN) || !Pattern.matches("[a-zA-Z/.0-9]+", packageN)) {
			updateStatus("Ohjelman tai packagen nimess ei voi olla merkkej");
			return;
		}

		if (packageN.lastIndexOf('.') == packageN.length() - 1) {
			updateStatus("Packagen nimi ei voi loppua pisteeseen");
			return;
		}

		if (packageN.indexOf('.') == 0) {
			updateStatus("Packagen nimi ei voi alkaa pisteell");
			return;
		}

		if (fileN.lastIndexOf('.') != -1) {
			updateStatus("Ohjelman nimess ei saa olla pistett");
			return;
		}

		updateStatus(null);
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	/**
	 * @return src:n polku
	 */
	public String getContainerName() {
		return containerText.getText();
	}

	/**
	 * @return ohjelman nimi
	 */
	public String getFileName() {
		return fileName.getText();
	}

	/**
	 * @return paketin nimi
	 */
	public String getPackageName() {
		return packageName.getText();
	}
}